/**
 * Handles the notification list on the dashboard.
 *
 * The `de_epages.shopUiDashboardNotifications` method creates acontroller for the notification list.
 *
 * ### Examples
 * Creates a controller to handle a notification list.
 *
 * JavaScript:
 *
 *     de_epages('.ep-dashboard-notification-container').shopUiDashboardNotifications({
 *       showMore: true #REM #ShowMore #ENDREM
 *     });
 *
 * HTML:
 *
 *     <ul class="ep-dashboard-notification-container">
 *       #IF(#Shop.ClosedByMerchant)
 *       <li data-notification-config='{"id": 12345, "deletable": false}' class="ep-dashboard-notification">
 *         <span class="ep-dashboard-icon fa fa-lock"></span>
 *         <span class="ep-dashboard-text">{ShopClosed} <a href="?ViewAction=UnityMBO-ViewStatusGeneral&ObjectID=#Shop.ID">{OpenShop}</a></span>
 *       </li>
 *       #ENDIF
 *       <li data-notification-config='{"id": 12345, "deletable": true}' class="ep-dashboard-notification">
 *         <span class="ep-dashboard-icon fa fa-clipboard"></span>
 *         <span class="ep-dashboard-text"><span class="ep-dashboard-text-highlight">1 Order</span> has been paid but needs to be fullfield. <a>View</a></span>
 *         <span class="ep-dashboard-cancel"></span>
 *       </li>
 *       <li data-notification-config='{"id": 12345, "deletable": true}' class="ep-dashboard-notification">
 *         <span class="ep-dashboard-icon fa fa-shopping-cart"></span>
 *         <span class="ep-dashboard-text"><span class="ep-dashboard-text-highlight">4 Products</span> are out of stock. <a>View</a></span>
 *         <span class="ep-dashboard-cancel fa fa-times"></span>
 *       </li>
 *       <li data-notification-config='{"id": 12345, "deletable": true}' class="ep-dashboard-notification">
 *         <span class="ep-dashboard-icon fa fa-cog"></span>
 *         <span class="ep-dashboard-text">PayPal Plus setup is not completed. <a>View</a></span>
 *         <span class="ep-dashboard-cancel fa fa-times"></span>
 *       </li>
 *       <li data-notification-config='{"id": 12345, "deletable": false}' class="ep-dashboard-notification">
 *         <span class="ep-dashboard-icon fa fa-cog"></span>
 *         <span class="ep-dashboard-text">PayPal Plus setup is not completed. <a>View</a></span>
 *       </li>
 *       <li data-notification-config='{"id": 12345, "deletable": true}' class="ep-dashboard-notification HideElement">
 *         <span class="ep-dashboard-icon fa fa-tag"></span>
 *         <span class="ep-dashboard-text">Your eBay connection will expire in 10 days. <a>View</a></span>
 *         <span class="ep-dashboard-cancel fa fa-times"></span>
 *       </li>
 *       <li data-notification-config='{"id": 12345, "deletable": true}' class="ep-dashboard-notification HideElement">
 *         <span class="ep-dashboard-icon fa fa-cog"></span>
 *         <span class="ep-dashboard-text">A PayPal transaction failed. <a>View</a></span>
 *         <span class="ep-dashboard-cancel fa fa-times"></span>
 *       </li>
 *       <li class="ep-dashboard-show-more">
 *         <span class="ep-dashboard-text"><a id="ep-dashboard-show-more-button">View All (<span id="ep-dashboard-hidden-notification-number">2</span> more)</a></span>
 *       </li>
 *     </ul>
 *
 *
 * @class jQuery.ui.shopUiDashboardNotifications
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses jQuery.ui.shopUiDashboardNotifications
 * @since 7.5.0
 */

/**
 * @cfg {Boolean} showMore Boolean Flag which indicates if the view more link should be shown.
 */

/**
 * See `jQuery.ui.shopUiDashboardNotifications` for details.
 *
 * @param {Object} options A map of additional options pass to the method.
 * @param {Boolean} howMore Boolean Flag which indicates if the view more link should be shown.
 *
 * @method shopUiDashboardNotifications
 * @member jQuery
 *
 * @since 7.5.0
 */

/*
 * @copyright       © Copyright 2006-2017, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.shop.ui.dashboard-notifications
 *
 * @revision        $Revision: 1.20 $
 */
/*globals define*/
/*jslint nomen: true*/

define('de_epages/shop/ui/dashboard-notifications', [
    'jquery',
    'ep',
    'de_epages',

    "jquery/ui/widget",
    "ep/fn/busy",
    "$ready!"
], function ($, ep, de_epages) {

    $.widget('ui.shopUiDashboardNotifications', {
        options: {
            showMore: false,
            selectors: {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                notificationRow: '.ep-dashboard-notification',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                removeButton: '.ep-dashboard-cancel'
            },
            removeChangeAction: "JSONMarkShopNotificationInactive"
        },

        _init: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                showMoreCounter = elem.find(o.selectors.hiddenRowNumber);

            // get all list elements
            self.listItems = elem.find(o.selectors.notificationRow);
            // get all initially hidden list elements
            self.hiddenListItems = self.listItems.filter('.HideElement');

            self.deletedItems = 0;

            if (o.showMore) {
                elem.find(o.selectors.showMoreButton).on('click', function() {
                    self.hiddenListItems.removeClass('HideElement');
                    self.hiddenListItems = [];

                    elem.find(o.selectors.showMoreRow).remove();
                });
            }

            self.listItems.each(function(index, li) {
                var $li = $(li),
                    config = $li.data('notificationConfig');

                if (config.deletable) {
                    $li.find(o.selectors.removeButton).on('click', function() {
                        $li.busy('show');
                        ep.ajax({
                            dataType: "json",
                            type : "POST",
                            data : {
                                ChangeAction : o.removeChangeAction,
                                ObjectID: config.id
                            }
                        })
                        .done(function( jsonData, statusText, jqXHR ){
                            $li.addClass('HideElement');
                            $li.busy('hide');

                            self.deletedItems++;

                            if (self.hiddenListItems.length > 2) {
                                self.hiddenListItems.eq(0).removeClass('HideElement');
                                self.hiddenListItems.splice(0, 1);
                                showMoreCounter.text(self.hiddenListItems.length);

                            } else if (self.hiddenListItems.length === 2) {
                                elem.find(o.selectors.showMoreRow).remove();
                                self.hiddenListItems.removeClass('HideElement');
                                self.hiddenListItems = [];

                            } else if (self.listItems.length === self.deletedItems) {
                                elem.find(o.selectors.emptyNotificationRow).removeClass('HideElement');
                            }
                        })
                        .fail(function() {
                            $li.busy('hide');
                        });

                    });
                }

            });

        },

        destroy: function () {
            var self = this;

            self._superApply(arguments);
        }

    });

    return de_epages;
});