/*jslint nomen:true, plusplus: true*/
/*global define*/

define("de_epages/mediagallery/filemanager/tree-view", [
    "backbone",
    "underscore",
    "jquery",
    "ep/dict",
    "$tmpl!./tree-view",
    "$dict!../dictionary",

    "jquery/tmpl",
    "jquery/jstree",
    "jquery/jstree/themes",
    "jquery/jstree/json_data"
], function (Backbone, _, $, ep, template, mediagalleryDict) {
    "use strict";

    /*
     * @dictionary       ep.dict
     *
     * @translation      {MediaGallery}
     *                   {BackgroundImages}
     *                   {HeaderImages}
     *                   {ImageDb}
     */

    var tMediaGallery = mediagalleryDict.translate("MediaGallery"),
        tBackgroundImages = mediagalleryDict.translate("BackgroundImages"),
        tHeaderImages = mediagalleryDict.translate("HeaderImages"),
        tImageDb = mediagalleryDict.translate("ImageDb"),

        view = Backbone.View.extend({
            tagName: "li",
            template: template,

            events: {
                "click .ep-mediagallery-treeHead": "onHeaderClick",
                "click .ep-mediagallery-tree a": "onLinkClick"
            },

            initialize: function (options) {
                this.options = options;

                var name = this.model.get("data"),
                    editable = (name === "MediaGallery") ? true : false,
                    state = (this.options.currentFolder === name) ? "open" : "closed";

                this.model.set({
                    state: state,
                    editable: editable
                });

                this.model
                    // show or hide a single tree
                    // possible states: open/closed
                    .on("change:state", this.setState, this)
                    // add a new node to the tree
                    .on("addTreeNode", this.addTreeNode, this)
                    // change the name of a tree node
                    .on("renameTreeNode", this.renameTreeNode, this)
                    // remove a tree node when a folder was deleted
                    .on("removeTreeNode", this.removeTreeNode, this)
                    // mark the currently selected node
                    .on("setActiveNode", this.setActiveNode, this);
            },

            render: function () {
                var self = this,
                    name = this.model.get("data"),
                    tName,
                    tmpl;

                // translate the given name
                switch (name) {
//#JSCOVERAGE_IF false
                case "MediaGallery":
                    tName = tMediaGallery;
                    break;
                case "HeaderImages":
                    tName = tHeaderImages;
                    break;
                case "BackgroundImages":
                    tName = tBackgroundImages;
                    break;
                case "ImageDB":
                    tName = tImageDb;
                    break;
//#JSCOVERAGE_ENDIF
                default:
                    break;
                }

                tmpl = this.template({
                    name: tName || name
                });
                this.setElement(tmpl);

                $.extend(this, $.tmplItem(tmpl).elements);

                _.each(self.options.treeData.children, function (item) {
                    if (item.metadata.displayname) {
                        item.data = item.metadata.displayname;
                    }

                    item.attr = {
                        id: item.metadata.type + item.metadata.path,
                        title: item.data
                    };
                });

                // create jstree
                this.tree.jstree({
                    "json_data": {
                        data: self.options.treeData.children,
//#JSCOVERAGE_IF false
                        ajax: {
                            url: '?',
                            type: 'POST',
                            dataType: 'json',
                            beforeSend: function (xhr, s) {
                                xhr.setRequestHeader('Accept', 'text/x-json');
                            },
                            context: this,
                            success: function (data) {
                                _.each(data[0].children, function (item) {
                                    if (item.metadata.displayname) {
                                        item.data = item.metadata.displayname;
                                    }

                                    item.attr = {
                                        id: item.metadata.type + item.metadata.path,
                                        title: item.data
                                    };
                                });
                                return data[0].children;
                            },
                            data: function (node) {
                                return {
                                    ViewAction: 'JSONGetDirectoryList',
                                    Type: node.data('jstree').type,
                                    ObjectID: ep.config.siteId,
                                    Directory: node.data('jstree').path,
                                    IgnoreDirectoryNotFound: 1
                                };
                            }
//#JSCOVERAGE_ENDIF
                        }
                    },
                    themes: {
                        theme: 'bo',
                        dots: true,
                        icons: true
                    },
                    "plugins": ["themes", "json_data", "crrm"]
                }).on("click", "a", function (event) {
                    var data = $(this).parent().data("jstree");

                    event.preventDefault();

                    if (self.prevNode !== this) {
                        self.trigger("openFolder", data);
                        self.prevNode = this;
                    }
                });

                if (this.model.get("state") === "closed") {
                    this.tree.hide();
                } else {
                    this.treeHead.addClass("active");
                }

                return this;
            },

            // create a new tree node when a new folder was created
            addTreeNode: function (node) {
                var id = (node.id === "MediaGallery") ? -1 : $("li[id='" + node.id + "']");

                node.data.attr = {
                    id: node.data.id,
                    title: node.data.data
                };
                this.tree.jstree("create_node", id, "last", node.data, false, true);

                // set jstree data because the create_node method obviously doens't do this
                this.$("li[id='" + node.data.id + "']").data("jstree", node.data.metadata);
            },

            // change the name of a tree node when a folder name was changed
            renameTreeNode: function (node) {
                this.$("li[id='" + node.prevID + "']").attr({
                    id: node.model.get("id"),
                    title: node.model.get("data")
                }).data("jstree", {
                    type: node.model.get("metadata").type,
                    path: node.model.get("metadata").path
                }).find("a:first").html('<ins class="jstree-icon">&nbsp;</ins>' + node.model.get("data"));
            },

            // remove a tree node when a folder was removed
            removeTreeNode: function (node) {
                this.tree.jstree("delete_node", this.$("li[id='" + node.id + "']"));
            },

            // mark the current folder as active
            setActiveNode: function (id) {
                this.tree.find("a.active").removeClass("active");
                $("li[id='" + id + "']").find("a:first").addClass("active");
            },

            // show or hide tree by clicking the header
            onHeaderClick: function () {
                if (!this.tree.is(":visible")) {
                    this.trigger("changeState");
                    this.model.set("state", "open");
                }

                this.trigger("folderEditable", this.model.get("editable"));
                this.trigger("openFolder", this.model.get("metadata"));
            },

            // handle click on jstree
            onLinkClick: function () {
                this.trigger("folderChanged", this.model);
            },

            // open or close accordion
            setState: function () {
                if (this.model.get("state") === "open") {
                    this.tree.show(500).find("a.active").removeClass("active");
                    this.treeHead.addClass("active");
                } else {
                    this.tree.hide(500);
                    this.treeHead.removeClass("active");
                }
            }
        });

    return view;
});