/**
 * Creates a multiselectbox forthe searchbar
 *
 * ### Examples
 * Create a simple multiselectbox.
 *
 *      <div id="test"></div>
 *
 *       <script>
 *           require([
 *             'jquery',
 *             'ep/ui/multiselect',
 *
 *             '$ready!'
 *           ], function($, ep) {
 *             ep('#test').uiMultiselect({
 *               statusSelect: {
 *                 tableHeader: '{Status}',
 *                 id: 'StatusSelect',y
 *                 values:[{
 *                   name: 'ViewedOn',
 *                   value: '{ViewedOn }'
 *                   #IF(#DEFINED(#INPUT.ViewedOn_checkbox)), checked: #INPUT.ViewedOn_checkbox#ENDIF
 *                   #IF(#DEFINED(#INPUT.ViewedOn)), selection: #INPUT.ViewedOn#ENDIF
 *                 },{
 *                   name: 'CancelledOn',
 *                   value: '{CancelledOn }'
 *                   #IF(#DEFINED(#INPUT.CancelledOn_checkbox)), checked: #INPUT.CancelledOn_checkbox#ENDIF
 *                   #IF(#DEFINED(#INPUT.CancelledOn)), selection: #INPUT.CancelledOn#ENDIF
 *                 }]
 *             });
 *           });
 *       </script>
 *
 *
 * @class jQuery.ui.presentationUiMultiselect
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses jQuery
 * @uses de_epages
 * @uses jQuery.ui.widget
 * @uses jQuery.dict
 * @uses jQuery.tmpl
 * @since 7.10.0
 */


/**
 * See `jQuery.ui.presentationUiMultiselect` for details.
 *
 * @param {Array} values        An Array of Objects configuring the selection
 * @param {String} id           A Unique to reference this instance
 * @param {String} tableHeader  Headline
 *
 * @method presentationUiMultiselect
 * @member jQuery
 *
 * @since 7.10.0
 */

/*
 * @copyright       © Copyright 2006-2017, epages GmbH, All Rights Reserved.
 *
 * @module          ep.ui.multiselect
 */

define("de_epages/presentation/ui/multiselect", [
    "jquery",
    "ep",
    "de_epages",
    '$tmpl!de_epages/presentation/ui/multiselect',
    "$dict!de_epages/presentation/dictionary",

    "jquery/ui/widget"
], function ($, ep, de_epages, tmplBase, dict) {

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Status}
     *                  {yes}
     *                  {no}
     *
     * @dictionary      de_epages.presentation.dictionary
     *
     * @translation     {UnitySelectedFilters}
     */

    var translations = {
        selected: dict.translate('UnitySelectedFilters'),
        status: dict.translate('Status'),
        yes: dict.translate('yes'),
        no: dict.translate('no')
    };

    $.widget( "ui.presentationUiMultiselect", {
        templateElements: {},
        options: {
            values: [],
            id: 'default',
            tableHeader: ''
        },
        open: false,
        selectionCount: 0,
        body: $('body'),

        _create: function(){
            var self = this,
                o = self.options;

            // get count of initially selected items
            self.selectionCount = o.values.reduce(function(accumulator, item) {
                if (item.checked) {
                    accumulator++;
                }
                return accumulator;
            }, 0);

            // init template
            self.multiselect = tmplBase({
                values: o.values,
                multiselectId: o.id,
                initialCount: self.selectionCount,
                tSelected: translations.selected,
                tableHeader: o.tableHeader,
                tYes: translations.yes,
                tNo: translations.no
            });

            // load template and push named tmpl elements to current instance
            $.extend(self.templateElements, self.multiselect.tmplItem('elements'));

            self.templateElements.selectionList.hide();

            // init events
            self.templateElements.selectionList.find('.ep-multiselect-checkbox').on('click.multiselect', {self: self}, self._handleCheckboxClick);
            self.templateElements.selectionButton.on('click.multiselect', {self: self}, self._handleButtonClick);
            self.templateElements.removeFilter.on('click.multiselectRemoveFilter', {self: self}, self._removeFilter);

            self.element.append(self.multiselect);
        },

        // remove filter
        _removeFilter: function(evt) {
            var self = evt.data.self;

            // loop over search filter
            self.templateElements.selectionList.find('.ep-multiselect-list-item').each(function(index, item) {
                var $checkbox = $(item).find('.ep-multiselect-checkbox');

                // reset checkbox and radio buttons
                $checkbox.prop('checked', false);
                self._disableOption($checkbox.data('id'));
            });

            // reset filter counter and hide remove filter button
            self.selectionCount = 0;
            self.templateElements.selectionCount.html(self.selectionCount);
            self.templateElements.removeFilter.hide();
        },

        // open / close the selectbox on click
        _handleButtonClick: function(evt) {
            var self = evt.data.self;

            if (self.open) {
                self.templateElements.selectionList.hide();
                self.body.off('click.multiselect');
            } else {
                self.templateElements.selectionList.show();
                self.body.on('click.multiselect', {self: self}, self._handleBodyClick);
            }

            self.open = !self.open;
        },

        // activate / disable the options on click on checkbox
        _handleCheckboxClick: function(evt) {
            var self = evt.data.self,
                checkbox = evt.target,
                $checkbox = $(checkbox),
                id = $checkbox.data('id'),
                value = self.options.values[id];

            if (checkbox.checked) {
                self.selectionCount++;
                self._activateOption(id);
            } else {
                self.selectionCount--;
                self._disableOption(id);
            }

            self.templateElements.selectionCount.html(self.selectionCount);
            // show/hide remove filter button
            self.templateElements.removeFilter[self.selectionCount > 0 ? 'show' : 'hide']();
        },

        // close dialog on click outside of box
        _handleBodyClick: function(evt) {
            var self = evt.data.self,
                clickPosition = {
                    x: evt.clientX,
                    y: evt.clientY
                },
                boxPosition = self.templateElements.selectionList[0].getBoundingClientRect();

            if (!(clickPosition.y > (boxPosition.top - self.templateElements.selectionButton.outerHeight(true))
                && clickPosition.y < boxPosition.bottom
                && clickPosition.x > boxPosition.left
                && clickPosition.x < boxPosition.right)) {

                self.body.off('click.multiselect');
                self.templateElements.selectionList.hide();
                self.open = false;
            }
        },

        // disable and uncheck radio buttons
        _disableOption: function(val) {
            var self = this;

            self.templateElements['listItem_' + val]
                .addClass('ep-multiselect-disabled')
                .find('input[type=radio]')
                    .prop({
                        'disabled': true,
                        'checked': false
                    });
        },

        // enable radio buttons
        _activateOption: function(val) {
            var self = this;

            self.templateElements['listItem_' + val]
                .removeClass('ep-multiselect-disabled')
                .find('input[type=radio]')
                    .prop('disabled', false)
                    .eq(0)
                    .prop('checked', true);
        },

        destroy: function(){
            var self = this;

            self._superApply(arguments);
        }

    });

    return de_epages;
});
